/*
 *  utils.c - Some useful things
 *  
 *  Copyright (C) 1999 Robert Cheramy <tibob@via.ecp.fr>
 */

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include <string.h>
#include <stdio.h>
#include <time.h>
/* #include <db.h> */
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "utils.h"


char *timefile(char *filemask, time_t when) {
  char s_temp[FILENAME_MAX];
  char *s_cpy;
  struct tm *p_time;
  
  p_time = localtime(&when);
  strftime(s_temp, FILENAME_MAX, filemask, p_time);

  s_cpy = strdup(s_temp);
  return (s_cpy);
}

int makedirname(char *filename) {
  static struct stat statbuf;
  char *s_temp = filename;
  char *s_slash = NULL;

  /* check in which directory `filename' is going to be,
   * so that we can get /foo/bar from /foo/bar/baz */
  while (*s_temp) {
    if('/' == *s_temp) {
      s_slash = s_temp;
    }
    s_temp++;
  }

  if (NULL == s_slash || s_slash == filename) {
    /* either `filename' is a relative path (pwd is a dir), or
     * `filename' is directly in /, which is a directory as well.
     * In both cases, return OK */
    return(0);
  }
  /* /foo/bar/baz becomes /foo/bar and we check if /foo/bar is a
   * directory, which is required to create baz afterwards */
  *s_slash = '\0';
  /* Check if /foo/bar is already a directory */
  if (-1 != stat(filename, &statbuf)) {
    if (S_ISDIR(statbuf.st_mode)) {
      *s_slash = '/';
      return(0);
    }
  }
  /* Check if we can create /foo/bar (recursive call to check /foo) */
  if (-1 != makedirname (filename)) {
    if (-1 != mkdir(filename, DEFAULT_DIR_MODE)) {
      *s_slash = '/';
      return(0);
    }
  }
  /* Everything failed, we give up */
  *s_slash = '/';
  return(-1);
}

